@@ -2,6 +2,10 @@
 #include "config.h"
 #endif
 
+#include <iostream>
+#include <fstream>
+#include <stdexcept>
+
 #include "rutil/Log.hxx"
 #include "rutil/Logger.hxx"
 #include "rutil/DnsUtil.hxx"
@@ -108,6 +112,7 @@ ReproRunner::ReproRunner()
    , mSipStack(0)
    , mStackThread(0)
    , mAbstractDb(0)
+   , mRuntimeAbstractDb(0)
    , mRegistrationPersistenceManager(0)
    , mAuthRequestDispatcher(0)
    , mAsyncProcessorDispatcher(0)
@@ -154,7 +159,8 @@ ReproRunner::run(int argc, char** argv)
    Data defaultConfigFilename("repro.config");
    try
    {
-      mProxyConfig = new ProxyConfig(mArgc, mArgv, defaultConfigFilename);
+      mProxyConfig = new ProxyConfig();
+      mProxyConfig->parseConfig(mArgc, mArgv, defaultConfigFilename);
    }
    catch(BaseException& ex)
    {
@@ -377,6 +383,7 @@ ReproRunner::cleanupObjects()
       delete mRegistrationPersistenceManager; mRegistrationPersistenceManager = 0;
    }
    delete mAbstractDb; mAbstractDb = 0;
+   delete mRuntimeAbstractDb; mRuntimeAbstractDb = 0;
    delete mStackThread; mStackThread = 0;
    delete mSipStack; mSipStack = 0;
    delete mAsyncProcessHandler; mAsyncProcessHandler = 0;
@@ -476,6 +483,19 @@ ReproRunner::createSipStack()
       mSipStack->setEnumSuffixes(enumSuffixes);
    }
 
+   // Set any enum domains from configuration
+   std::map<Data,Data> enumDomains;
+   std::vector<Data> _enumDomains;
+   mProxyConfig->getConfigValue("EnumDomains", _enumDomains);
+   if (enumSuffixes.size() > 0)
+   {
+      for(std::vector<Data>::iterator it = _enumDomains.begin(); it != _enumDomains.end(); it++)
+      {
+         enumDomains[*it] = *it;
+      }
+      mSipStack->setEnumDomains(enumDomains);
+   }
+
    // Add stack transports
    bool allTransportsSpecifyRecordRoute=false;
    if(!addTransports(allTransportsSpecifyRecordRoute))
@@ -581,10 +601,11 @@ ReproRunner::createDatastore()
 {
    // Create Database access objects
    assert(!mAbstractDb);
+   assert(!mRuntimeAbstractDb);
 #ifdef USE_MYSQL
    Data mySQLServer;
    mProxyConfig->getConfigValue("MySQLServer", mySQLServer);
-   if (!mySQLServer.empty())
+   if(!mySQLServer.empty())
    {
       mAbstractDb = new MySqlDb(mySQLServer, 
                        mProxyConfig->getConfigData("MySQLUser", ""), 
@@ -593,6 +614,17 @@ ReproRunner::createDatastore()
                        mProxyConfig->getConfigUnsignedLong("MySQLPort", 0),
                        mProxyConfig->getConfigData("MySQLCustomUserAuthQuery", ""));
    }
+   Data runtimeMySQLServer;
+   mProxyConfig->getConfigValue("RuntimeMySQLServer", runtimeMySQLServer);
+   if(!runtimeMySQLServer.empty())
+   {
+      mRuntimeAbstractDb = new MySqlDb(runtimeMySQLServer,
+                       mProxyConfig->getConfigData("RuntimeMySQLUser", ""), 
+                       mProxyConfig->getConfigData("RuntimeMySQLPassword", ""),
+                       mProxyConfig->getConfigData("RuntimeMySQLDatabaseName", ""),
+                       mProxyConfig->getConfigUnsignedLong("RuntimeMySQLPort", 0),
+                       mProxyConfig->getConfigData("MySQLCustomUserAuthQuery", ""));
+   }
 #endif
    if (!mAbstractDb)
    {
@@ -605,7 +637,13 @@ ReproRunner::createDatastore()
       cleanupObjects();
       return false;
    }
-   mProxyConfig->createDataStore(mAbstractDb);
+   if(mRuntimeAbstractDb && !mRuntimeAbstractDb->isSane())
+   {
+      CritLog(<<"Failed to open runtime configuration database");
+      cleanupObjects();
+      return false;
+   }
+   mProxyConfig->createDataStore(mAbstractDb, mRuntimeAbstractDb);
 
    // Create ImMemory Registration Database
    mRegSyncPort = mProxyConfig->getConfigInt("RegSyncPort", 0);
@@ -699,13 +737,18 @@ ReproRunner::createDialogUsageManager()
 
    if (mDum)
    {
-      if(mProxyConfig->getConfigBool("EnableCertificateAuthenticator", false))
+      bool enableCertAuth = mProxyConfig->getConfigBool("EnableCertificateAuthenticator", false);
+      // Maintains existing behavior for non-TLS cert auth users
+      bool digestChallengeThirdParties = !enableCertAuth;
+
+      if(enableCertAuth)
       {
          // TODO: perhaps this should be initialised from the trusted node
          // monkey?  Or should the list of trusted TLS peers be independent
          // from the trusted node list?
          std::set<Data> trustedPeers;
-         SharedPtr<TlsPeerAuthManager> certAuth(new TlsPeerAuthManager(*mDum, mDum->dumIncomingTarget(), trustedPeers));
+         loadCommonNameMappings();
+         SharedPtr<TlsPeerAuthManager> certAuth(new TlsPeerAuthManager(*mDum, mDum->dumIncomingTarget(), trustedPeers, true, mCommonNameMappings));
          mDum->addIncomingFeature(certAuth);
       }
 
@@ -726,7 +769,8 @@ ReproRunner::createDialogUsageManager()
                                                 mAuthRequestDispatcher,
                                                 mProxyConfig->getDataStore()->mAclStore,
                                                 !mProxyConfig->getConfigBool("DisableAuthInt", false) /*useAuthInt*/,
-                                                mProxyConfig->getConfigBool("RejectBadNonces", false)));
+                                                mProxyConfig->getConfigBool("RejectBadNonces", false),
+                                                digestChallengeThirdParties));
          mDum->setServerAuthManager(uasAuth);
       }
 
@@ -937,6 +981,9 @@ ReproRunner::addDomains(TransactionUser& tu, bool log)
       }
    }
 
+   /* All of this logic has been commented out - the sysadmin must explicitly
+      add any of the items below to the Domains config option in repro.config
+
    Data localhostname(DnsUtil::getLocalHostName());
    if(log) InfoLog (<< "Adding local hostname domain " << localhostname );
    tu.addDomain(localhostname);
@@ -960,7 +1007,10 @@ ReproRunner::addDomains(TransactionUser& tu, bool log)
    }
 
    if(log) InfoLog (<< "Adding 127.0.0.1 domain.");
-   tu.addDomain("127.0.0.1");
+   tu.addDomain("127.0.0.1"); */
+
+   if( realm.empty() )
+      realm = "Unconfigured";
 
    return realm;
 }
@@ -1176,6 +1226,72 @@ ReproRunner::addTransports(bool& allTransportsSpecifyRecordRoute)
    return true;
 }
 
+void 
+ReproRunner::addProcessor(repro::ProcessorChain& chain, std::auto_ptr<Processor> processor)
+{
+   chain.addProcessor(processor);
+}
+
+void
+ReproRunner::loadCommonNameMappings()
+{
+   // Already loaded?
+   if(!mCommonNameMappings.empty())
+      return;
+
+   Data mappingsFileName = mProxyConfig->getConfigData("CommonNameMappings", "");
+   if(mappingsFileName.empty())
+      return;
+
+   InfoLog(<< "trying to load common name mappings from file: " << mappingsFileName);
+
+   ifstream mappingsFile(mappingsFileName.c_str());
+   if(!mappingsFile)
+   {
+      throw std::runtime_error("Error opening/reading mappings file");
+   }
+
+   string sline;
+   while(getline(mappingsFile, sline))
+   {
+      Data line(sline);
+      Data cn;
+      PermittedFromAddresses permitted;
+      ParseBuffer pb(line);
+
+      pb.skipWhitespace();
+      const char * anchor = pb.position();
+      if(pb.eof() || *anchor == '#') continue;  // if line is a comment or blank then skip it
+
+      // Look for end of name
+      pb.skipToOneOf("\t");
+      pb.data(cn, anchor);
+      pb.skipChar('\t');
+
+      while(!pb.eof())
+      {
+         pb.skipWhitespace();
+         if(pb.eof())
+            continue;
+
+         Data value;
+         anchor = pb.position();
+         pb.skipToOneOf(",\r\n ");
+         pb.data(value, anchor);
+         if(!value.empty())
+         {
+            StackLog(<< "Loading CN '" << cn << "', found mapping '" << value << "'");
+            permitted.insert(value);
+         }
+         if(!pb.eof())
+            pb.skipChar();
+      }
+
+      DebugLog(<< "Loaded mapping for CN '" << cn << "', " << permitted.size() << " mapping(s)");
+      mCommonNameMappings[cn] = permitted;
+   }
+}
+
 void  // Monkeys
 ReproRunner::makeRequestProcessorChain(ProcessorChain& chain)
 {
@@ -1183,10 +1299,10 @@ ReproRunner::makeRequestProcessorChain(ProcessorChain& chain)
    assert(mRegistrationPersistenceManager);
 
    // Add strict route fixup monkey
-   chain.addProcessor(std::auto_ptr<Processor>(new StrictRouteFixup));
+   addProcessor(chain, std::auto_ptr<Processor>(new StrictRouteFixup));
 
    // Add is trusted node monkey
-   chain.addProcessor(std::auto_ptr<Processor>(new IsTrustedNode(*mProxyConfig)));
+   addProcessor(chain, std::auto_ptr<Processor>(new IsTrustedNode(*mProxyConfig)));
 
    // Add Certificate Authenticator - if required
    if(mProxyConfig->getConfigBool("EnableCertificateAuthenticator", false))
@@ -1197,7 +1313,8 @@ ReproRunner::makeRequestProcessorChain(ProcessorChain& chain)
       // Should we used the same trustedPeers object that was
       // passed to TlsPeerAuthManager perhaps?
       std::set<Data> trustedPeers;
-      chain.addProcessor(std::auto_ptr<Processor>(new CertificateAuthenticator(*mProxyConfig, mSipStack, trustedPeers)));
+      loadCommonNameMappings();
+      addProcessor(chain, std::auto_ptr<Processor>(new CertificateAuthenticator(*mProxyConfig, mSipStack, trustedPeers, true, mCommonNameMappings)));
    }
 
    // Add digest authenticator monkey - if required
@@ -1206,18 +1323,18 @@ ReproRunner::makeRequestProcessorChain(ProcessorChain& chain)
       assert(mAuthRequestDispatcher);
       DigestAuthenticator* da = new DigestAuthenticator(*mProxyConfig, mAuthRequestDispatcher);
 
-      chain.addProcessor(std::auto_ptr<Processor>(da)); 
+      addProcessor(chain, std::auto_ptr<Processor>(da)); 
    }
 
    // Add am I responsible monkey
-   chain.addProcessor(std::auto_ptr<Processor>(new AmIResponsible)); 
+   addProcessor(chain, std::auto_ptr<Processor>(new AmIResponsible)); 
 
    // Add RequestFilter monkey
    if(!mProxyConfig->getConfigBool("DisableRequestFilterProcessor", false))
    {
       if(mAsyncProcessorDispatcher)
       {
-         chain.addProcessor(std::auto_ptr<Processor>(new RequestFilter(*mProxyConfig, mAsyncProcessorDispatcher)));
+         addProcessor(chain, std::auto_ptr<Processor>(new RequestFilter(*mProxyConfig, mAsyncProcessorDispatcher)));
       }
       else
       {
@@ -1235,16 +1352,16 @@ ReproRunner::makeRequestProcessorChain(ProcessorChain& chain)
    if (routeSet.empty())
    {
       // add static route monkey
-      chain.addProcessor(std::auto_ptr<Processor>(new StaticRoute(*mProxyConfig))); 
+      addProcessor(chain, std::auto_ptr<Processor>(new StaticRoute(*mProxyConfig))); 
    }
    else
    {
       // add simple static route monkey
-      chain.addProcessor(std::auto_ptr<Processor>(new SimpleStaticRoute(*mProxyConfig))); 
+      addProcessor(chain, std::auto_ptr<Processor>(new SimpleStaticRoute(*mProxyConfig))); 
    }
 
    // Add location server monkey
-   chain.addProcessor(std::auto_ptr<Processor>(new LocationServer(*mProxyConfig, *mRegistrationPersistenceManager, mAuthRequestDispatcher)));
+   addProcessor(chain, std::auto_ptr<Processor>(new LocationServer(*mProxyConfig, *mRegistrationPersistenceManager, mAuthRequestDispatcher)));
 
    // Add message silo monkey
    if(mProxyConfig->getConfigBool("MessageSiloEnabled", false))
@@ -1253,7 +1370,7 @@ ReproRunner::makeRequestProcessorChain(ProcessorChain& chain)
       {
          MessageSilo* silo = new MessageSilo(*mProxyConfig, mAsyncProcessorDispatcher);
          mRegistrar->addRegistrarHandler(silo);
-         chain.addProcessor(std::auto_ptr<Processor>(silo));
+         addProcessor(chain, std::auto_ptr<Processor>(silo));
       }
       else
       {
@@ -1267,13 +1384,14 @@ ReproRunner::makeResponseProcessorChain(ProcessorChain& chain)
 {
    assert(mProxyConfig);
    assert(mRegistrationPersistenceManager);
+
    // Add outbound target handler lemur
-   chain.addProcessor(std::auto_ptr<Processor>(new OutboundTargetHandler(*mRegistrationPersistenceManager))); 
+   addProcessor(chain, std::auto_ptr<Processor>(new OutboundTargetHandler(*mRegistrationPersistenceManager))); 
 
    if (mProxyConfig->getConfigBool("RecursiveRedirect", false))
    {
       // Add recursive redirect lemur
-      chain.addProcessor(std::auto_ptr<Processor>(new RecursiveRedirect)); 
+      addProcessor(chain, std::auto_ptr<Processor>(new RecursiveRedirect)); 
    }
 }
 
@@ -1285,18 +1403,18 @@ ReproRunner::makeTargetProcessorChain(ProcessorChain& chain)
 #ifndef RESIP_FIXED_POINT
    if(mProxyConfig->getConfigBool("GeoProximityTargetSorting", false))
    {
-      chain.addProcessor(std::auto_ptr<Processor>(new GeoProximityTargetSorter(*mProxyConfig)));
+      addProcessor(chain, std::auto_ptr<Processor>(new GeoProximityTargetSorter(*mProxyConfig)));
    }
 #endif
 
    if(mProxyConfig->getConfigBool("QValue", true))
    {
       // Add q value target handler baboon
-      chain.addProcessor(std::auto_ptr<Processor>(new QValueTargetHandler(*mProxyConfig))); 
+      addProcessor(chain, std::auto_ptr<Processor>(new QValueTargetHandler(*mProxyConfig))); 
    }
    
    // Add simple target handler baboon
-   chain.addProcessor(std::auto_ptr<Processor>(new SimpleTargetHandler)); 
+   addProcessor(chain, std::auto_ptr<Processor>(new SimpleTargetHandler)); 
 }
 
 
