@@ -925,6 +925,12 @@ SipStack::setEnumSuffixes(const std::vector<Data>& suffixes)
    mDnsStub->setEnumSuffixes(suffixes);
 }
 
+void
+SipStack::setEnumDomains(const std::map<Data,Data>& domains)
+{
+   mDnsStub->setEnumDomains(domains);
+}
+
 void
 SipStack::clearDnsCache()
 {
